package com.steerinfo.ftp.uploadfile.utils;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPConnectionClosedException;
import org.apache.commons.net.ftp.FTPReply;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.*;

@Component
public class FtpFileUtil {
    //ftp服务器ip地址
    @Value("${custom.config.file-server.ip}") // 相关配置放在application.properties 中
     String FTP_ADDRESS;
    //端口号
    @Value("${custom.config.file-server.port}")
    Integer FTP_PORT;
    //用户名
    @Value("${custom.config.file-ftp-user}")
    String FTP_USERNAME;
    //密码
    @Value("${custom.config.file-ftp-password}")
    String FTP_PASSWORD;
    //图片路径
    @Value("${custom.config.file-savepath}")
    String FTP_BASEPATH;

    public String getFTP_BASEPATH() {
        return FTP_BASEPATH;
    }

    public void setFTP_BASEPATH(String FTP_BASEPATH) {
        this.FTP_BASEPATH = FTP_BASEPATH;
    }

    private FTPClient ftpClient = new FTPClient();
      //上传
//    public boolean uploadFile(String filePath, String filename,String basePath, InputStream inputStream) {
//        boolean result = false;
//        try {
//            // 连接FTP服务器
//            ftpClient.setControlEncoding("UTF-8");
//            ftpClient.enterLocalPassiveMode();
//            ftpClient.connect(FTP_ADDRESS, FTP_PORT);
//            ftpClient.login(FTP_USERNAME, FTP_PASSWORD);
//            int reply;
//            reply = ftpClient.getReplyCode();
//            if (!FTPReply.isPositiveCompletion(reply)) {
//                ftpClient.disconnect();
//                return result;
//            }
//            //切换到上传目录
//            boolean changed = ftpClient.changeWorkingDirectory( FTP_BASEPATH+ filePath);
//            if (!changed) {
//                //如果目录不存在创建目录
//                String[] dirs = filePath.split("/");
//                String tempPath = basePath;
//                for (String dir : dirs) {
//                    if (null == dir || "".equals(dir)){
//                        continue;
//                    }
//                    tempPath += "/" + dir;
//                        if (!ftpClient.makeDirectory(tempPath)) {
//                            return result;
//                        } else {
//                            ftpClient.changeWorkingDirectory(tempPath);
//                        }
//                }
//            }
//            System.out.println("当前目录" + ftpClient.printWorkingDirectory());
//            //设置为被动模式
//            // ftp.enterLocalPassiveMode();
//            ftpClient.setControlEncoding("UTF-8");
//            //设置上传文件的类型为二进制类型
//            ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
//            //上传文件
//            if (!ftpClient.storeFile(new String(filename.getBytes("UTF-8"), "ISO-8859-1"), inputStream)) {
//                System.out.println("上传文件失败");
//                return result;
//            }
//            ftpClient.logout();
//            result = true;
//        } catch (IOException e) {
//            e.printStackTrace();
//        } finally {
//            if (ftpClient.isConnected()) {
//                try {
//                    inputStream.close();
//                    ftpClient.disconnect();
//                } catch (IOException ioe) {
//                    ioe.getMessage();
//                }
//            }
//        }
//        return result;
//    }

    //上传
    public boolean uploadToFtp(InputStream buffIn,String filePath, String fileName, boolean needDelete)
            throws FTPConnectionClosedException, IOException, Exception{
        boolean result = false;
        try {
            //建立连接
            connectToServer();
            setFileType(FTP.BINARY_FILE_TYPE);
            int reply = ftpClient.getReplyCode();
            if (!FTPReply.isPositiveCompletion(reply)) {
                ftpClient.disconnect();
                throw new IOException("failed to connect to the FTP Server:" + FTP_ADDRESS);
            }
            //进入文件目录
            boolean changeWork = ftpClient.changeWorkingDirectory( FTP_BASEPATH+ filePath);
            if(!changeWork){
                if(!createDirectory(FTP_BASEPATH+filePath)){
                    return result;
                }
            }
            result = ftpClient.storeFile(new String(fileName.getBytes("UTF-8"), "ISO-8859-1"),buffIn);
            if(needDelete){
                ftpClient.deleteFile(fileName);
            }
            // 输出操作结果信息
            if (result) {
                System.out.println("uploadToFtp INFO: upload file  to ftp : succeed!");
            } else {
                System.out.println("uploadToFtp INFO: upload file  to ftp : failed!");
            }
            buffIn.close();
        } catch (FTPConnectionClosedException e) {
            System.out.println("ftp连接被关闭！");
            throw e;
        }catch (Exception e){
            result = false;
            System.out.println("ERR : upload file  to ftp : failed! ");
            throw  e;
        } finally {
            try {
                if(buffIn != null){
                    buffIn.close();
                }
            } catch (Exception e){
                System.out.println("ftp关闭输入流时失败！");
            }
            if(ftpClient.isConnected()){
              closeConnect();
            }
        }
        return  result;
    }

    //下载
    public InputStream downloadFile(String filename)
            throws IOException {
        InputStream inputStream = null;
        try {
            connectToServer();
            ftpClient.enterLocalPassiveMode();
            // 设置传输二进制文件
            setFileType(FTP.BINARY_FILE_TYPE);
            int reply = ftpClient.getReplyCode();
            if(!FTPReply.isPositiveCompletion(reply)){
                ftpClient.disconnect();
                throw new IOException("failed to connect to the FTP Server:" + FTP_ADDRESS);
            }
            ftpClient.changeWorkingDirectory("/ems/file/2021/07/27");
            File file =  new File("D:\\LRL"+File.separatorChar+filename);
            OutputStream os = new FileOutputStream(file);
            boolean b = ftpClient.retrieveFile(filename, os);
            if(b == true){
                os.close();
                closeConnect();
            }
        } catch (FTPConnectionClosedException e){
            System.out.println("ftp连接被关闭！");
            throw e;
        } catch (Exception e) {
            System.out.println("ERR : upload file " + filename + " from ftp : failed!");
        }
        return inputStream;
    }

    //建立连接
    private void connectToServer() throws FTPConnectionClosedException, Exception {
         if(!ftpClient.isConnected()){
             int reply;
             try {
                //建立连接
                 ftpClient = new FTPClient();
                 ftpClient.setControlEncoding("UTF-8");
                 ftpClient.enterLocalPassiveMode();
                 ftpClient.connect(FTP_ADDRESS, FTP_PORT);
                 ftpClient.login(FTP_USERNAME, FTP_PASSWORD);
                 reply = ftpClient.getReplyCode();
                 if(!FTPReply.isPositiveCompletion(reply)){
                     ftpClient.disconnect();
                     System.out.println("connectToServer FTP server refused connection.");
                 }
             } catch (FTPConnectionClosedException ex) {
                 System.out.println("没有连接数！there are too many connected users,please try later");
                 throw  ex;
             } catch (Exception e) {
                 System.out.println("登录ftp服务器失败");
                 throw e;
             }
         }
    }

    //设置传输文件类型
    private void setFileType(int fileType) {
        try {
            ftpClient.setFileType(fileType);
        } catch (Exception e) {
            System.out.println("ftp设置传输文件的类型时失败！");

        }
    }

    //关闭连接
    public void closeConnect() {
        try {
            if (ftpClient != null) {
                ftpClient.logout();
                ftpClient.disconnect();
            }
        } catch (Exception e) {
            System.out.println("ftp连接关闭失败！");
        }
    }

   //创建文件夹
   public boolean createDirectory(String directory) throws IOException {
       boolean isSuccess = false;
       try {
               String[] dirs = directory.split("/");
               String tempPath = "";
               for (String dir : dirs) {
                   if(null == dir || "".equals(dir)){
                       continue;
                   }
                   tempPath += "/" + dir;
                   if (!ftpClient.makeDirectory(tempPath)) {
                       return isSuccess;
                   } else {
                       ftpClient.changeWorkingDirectory(tempPath);
                   }
               }
       } catch (Exception e) {
           e.printStackTrace();
       }
           return true;
   }
    //获取类型地址
//    public String getFtpaddress (String type) {
//        if(){}
//    }
}
